# -*- coding: utf-8 -*-

import os
import traceback

from lxml import etree as etree
from upload.converter.base_converter import (Converter)
from upload.converter.smagPlugins.xml.converter_ukm_xml import ConverterUkm

errorObjectsCount = 0


class SmagUkmXMLConverter(Converter):
    tnvds = dict()

    def __init__(self, node):
        super(SmagUkmXMLConverter, self).__init__(node)
        self.work_dir = None
        self.errorList = list()
        self.filePosition = 0
        self.currentFileSize = 0
        self.currentFile = None
        self.objects = []
        self.formatter = ConverterUkm()
        self.parser = {
            "updateItems": lambda arg: self.parse_invent(**arg),
            "storePrices": lambda arg: self.parse_price(**arg),
            "storePricesAdd": lambda arg: self.parse_tprice(**arg),
            "taxes": lambda arg: self.parse_tax(**arg),
            "classif": lambda arg: self.parse_group(**arg),
            "kit": lambda arg: self.parse_kit(**arg)
        }
        self.formatter.tnvdMapper = node.currentConverter.tnvdMapper
        self.formatter.measureMapper = node.currentConverter.measureMapper

    def parse(self, data):
        """
        @override

        Метод парсинга всех файлов
        data - список файлов
        Возвращает результат вставки (aif-объектов, список ошибок)
        """
        if not self.inserter.start():
            self.log.error(u'Не удалось запустить Inserter')
            return False, self.errorList
        try:
            for filePath in data:
                if os.stat(filePath).st_size == 0:
                    self.log.warn("Обнаружен пустой файл: %s" % filePath)
                    continue
                self.parseSingleFile(filePath)
        except Exception as e:
            print(e)
            self.log.error(str(e) + (u'(позиция в файле %s)' % self.filePosition))
            self.log.error(traceback.format_exc())
            self.errorList.append(e)
            return False, self.errorList
        result = self.inserter.stop()
        global errorObjectsCount
        errorObjectsCount = self.inserter.invalidObjectsCount()
        return result, self.errorList

    def parseSingleFile(self, file_path):
        """
        Парсинг данных из xml-файла
        filePath - путь до файла
        """

        # iterparse тут может выглядеть излишним, но позволяет считать root-ноду без считывания всего дерева
        args = {"file_path": file_path}
        events = ("start",)
        context = etree.iterparse(file_path, events=events)
        action, elem = next(context)
        tag = elem.tag
        if tag == "storePrices" or tag == "storePricesAdd":
            args["shopcode"] = elem.attrib["storeId"]
        self.clear_element(elem)
        del context
        if tag != 'CodeTNVED' and tag in self.parser:
            self.parser[tag](args)
        elif tag != 'CodeTNVED':
            self.log.warn(u"Работа с тегом %s не поддержана" % tag)

    def parse_invent(self, file_path):
        """
        парсинг товаров из xml
        """
        alcocodes = list()
        tmcs = list()
        context = etree.iterparse(file_path)
        for action, elem in context:
            if action == "end" and elem.tag == "item":
                tmc = {x.tag: x.text for x in elem}
                # считываем список ШК товара из xml
                barcodes_xml = elem.findall('barcode')
                barcodes = list()
                if len(barcodes_xml) > 0:
                    for bcode in barcodes_xml:
                        barcodes.append({x.tag: x.text for x in bcode})
                tmc['barcode'] = barcodes
                # считываем список алкокодов товара из xml
                if elem.find('ProductAlcCodes') is not None:
                    for alc_code in elem.find('ProductAlcCodes').getchildren():
                        # для алкокодов дополнительно добавляем код товара, чтобы заполнить таблицу tmcalcocode
                        alcocodes.append({alc_code.tag: alc_code.text, "article": tmc['article']})
                self.clear_element(elem)
                tmcs.append(tmc)
                tmc = None
        del context
        tnvds = self.get_tnvd()
        self.load_elements("updateItems", tmcs, {"tnvds": tnvds})
        self.load_elements("alcocode", alcocodes)

    def parse_price(self, file_path, shopcode):
        """
        парсинг переоценок товаров из xml
        """
        self.log.debug("Загрузка справочника prices")
        if shopcode != self.shopcode:
            self.log.warn("Код магазина в справочнике не совпадает с текущим, справочник не будет загружен!")
            return
        prices_list = list()
        context = etree.iterparse(file_path)
        for action, elem in context:
            if action == "end" and elem.tag == "item":
                article = elem.attrib['article']
                prices = elem.findall('price')
                for pr in prices:
                    price = {}
                    mrc = pr.find('MRC')
                    value = pr.find('value')
                    minprice = pr.find('minprice')
                    price[value.tag] = value.text
                    price[minprice.tag] = minprice.text
                    price['article'] = article
                    max_mrc = max(
                        list(map(lambda y: y['MRCValue'], [{mrc.tag: mrc.text} for mrc in mrc.getchildren()]))) \
                        if (mrc is not None) else 0
                    price['mrc'] = max_mrc
                    prices_list.append(price)
                self.clear_element(elem)
        del context
        self.load_elements("storePrices", prices_list)

    def parse_tprice(self, file_path, shopcode):
        """
        парсинг сущности tprice из xml
        """
        self.log.debug("Загрузка справочника tprices")
        if shopcode != self.shopcode:
            self.log.warn("Код магазина в справочнике не совпадает с текущим, справочник не будет загружен!")
            return
        price_type_code = ""
        price_type_name = ""
        prices_list = list()
        context = etree.iterparse(file_path)
        for action, elem in context:
            if action == "end" and elem.tag == "item":
                article = elem.attrib['article']
                for pr in elem:
                    price_value = pr.findtext('value')
                    """
                    берем максимальную МРЦ из списка
                    в качестве итоговой цены берем максимум между value из товара и МРЦ
                    если нет МРЦ сравниваем value с minprice
                    """
                    mrc = pr.find('MRC')
                    max_mrc = max(
                        list(map(lambda y: y['MRCValue'], [{mrc.tag: mrc.text} for mrc in mrc.getchildren()]))) \
                        if mrc is not None else pr.findtext('minprice')
                    price = {'value': max(float(price_value), float(max_mrc)), 'article': article,
                             'code': price_type_code,
                             'name': price_type_name}
                    prices_list.append(price)
                    self.clear_element(elem)
            elif action == "end" and elem.tag == "priceTypeCode":
                price_type_code = elem.text
                self.clear_element(elem)
            elif action == "end" and elem.tag == "priceTypeName":
                price_type_name = elem.text
                self.clear_element(elem)
        del context
        self.load_elements("storePricesAdd", prices_list)

    def parse_group(self, file_path):
        """
        парсинг групп товаров из xml
        """
        groups = list()
        context = etree.iterparse(file_path)
        for action, elem in context:
            if action == "end" and elem.tag == "group":
                group = {x.tag: x.text for x in elem}
                groups.append(group)
                self.clear_element(elem)
        del context
        self.load_elements("classif", groups)

    def parse_kit(self, file_path):
        """
        парсинг комплектов товаров из xml
        """
        kits = list()
        context = etree.iterparse(file_path)
        for action, elem in context:
            if action == "end" and elem.tag == "item":
                article = elem.attrib["article"]
                kit_items = [{"article": item.findtext("article"),
                              "quantity": item.findtext("quantity"),
                              'parentcode': article} for item in elem.findall("subItem")]
                kits.append({'article': article, 'subItem': kit_items})
                del kit_items
                self.clear_element(elem)
        del context
        self.load_elements("kit", kits)

    def parse_tax(self, file_path):
        """
        парсинг налогов из xml
        """
        taxes = list()
        context = etree.iterparse(file_path)
        for action, elem in context:
            if action == "end" and elem.tag == "group":
                code = elem.findtext('id')
                for tax in elem.findall("tax"):
                    tax_dict = dict()
                    for t in tax.getchildren():
                        tax_dict[t.tag] = t.text
                    tax_dict["id"] = code
                    taxes.append(tax_dict)
                self.clear_element(elem)
        del context
        self.load_elements("taxes", taxes)

    def get_tnvd(self):
        """
        чтение кодов тнвд из файла для последующего маппинга поля tmcType
        """
        tnvd_map = {}
        for (root, dirs, files) in os.walk(self.work_dir, topdown=True):
            for f in list(filter(lambda x: x.endswith('.xml'), files)):
                filepath = os.path.normpath(root + "/" + f)
                # ищем нужный root-узел
                context = etree.iterparse(filepath, events=("start",))
                action, elem = next(context)
                tag = elem.tag
                elem.clear()
                del elem
                del context
                if tag == 'CodeTNVED':
                    self.log.info("Найден файл, содержащий коды ТНВД: %s" % filepath)
                    tnvd_context = etree.iterparse(filepath)
                    for event, el in tnvd_context:
                        if event == "end" and el.tag == "group":
                            tnvd_map[el.findtext('id')] = str(el.findtext('code')).replace("\n", "").replace(" ", "")
                            self.clear_element(el)
                    del tnvd_context
        if len(tnvd_map) == 0:
            self.log.info("Не найден файл, содержащий коды ТНВД")
        return tnvd_map

    def clear_element(self, elem):
        elem.clear()
        # удаляем предыдущие считанные объекты
        while elem.getprevious() is not None:
            del elem.getparent()[0]
        del elem

    def load_elements(self, tag, data, kwargs=None):
        """
        Маппинг и загрузка элементов в nes
        загрузка осуществляется партиями (для экономии памяти)
        """
        part_size = 5000
        if kwargs is None:
            kwargs = {}
        if len(data) > 0:
            data_partial_list = [data[i:i + part_size] for i in range(0, len(data), part_size)]
            data_parts = [list(part) for part in data_partial_list]
            for part in data_parts:
                add_commands = self.formatter.get_command(tag, part, kwargs)  # type: list
                self.load_objects(list(add_commands), len(add_commands))
                del add_commands
            del data_parts
            del data_partial_list


def invalidObjectsCount():
    return errorObjectsCount


def general_method(workDirectory, data, node):
    converter = SmagUkmXMLConverter(node)
    converter.work_dir = workDirectory
    return converter.parse(data)
